(function () {
  function hasClass(elem, classes) {
    if (typeof classes === 'string') {
      classes = [classes];
    }
    var elem_classes = elem.className.split(' ');

    for (var c = 0; c < classes.length; c++) {
      if (elem_classes.indexOf(classes[c]) < 0) {
        return false;
      }
    }

    return true;
  }

  function forChildren(elem, func) {
    if (elem && elem.children && elem.children.length) {
      for (var c = 0; c < elem.children.length; c++) {
        if (elem.children[c] !== elem) {
          func(elem.children[c], c, elem.children);
        }
      }
    }
  }

  function forSiblings(elem, func) {
    if (elem) {
      forChildren(elem.parentElement, func);
    }
  }

  function getHeirarchy(elem) {
    var heirarchy = [];

    while (elem) {
      heirarchy.push(elem);
      elem = elem.parentElement;
    }

    return heirarchy;
  }

  function activateMenu(elem) {
    var touched = [], close = [].slice.call(document.querySelectorAll('.dropdown-menus, .dropdown-menus .block-menu, .dropdown-menus .menu, .dropdown-menus .menu-item.has-submenu, .dropdown-menus .menu-link.has-submenu'));

    var canOpen = true;

    getHeirarchy(elem).forEach(function (elem) {
      if (hasClass(elem, 'menu')) {
        if (canOpen) {
          elem.classList.add('open');
        } else {
          canOpen = true;
          elem.classList.remove('open');
        }

        elem.setAttribute('aria-hidden', false);
        touched.push(elem);

        forSiblings(elem, function (sib) {
          touched.push(sib);

          if (hasClass(sib, ['menu-link', 'has-submenu'])) {
            sib.classList.add('open');
          }
        });
      } else if (hasClass(elem, ['menu-link', 'has-submenu'])) {
        canOpen = true;
        elem.classList.add('open');
        touched.push(elem);
        forSiblings(elem, function (sib) {
          touched.push(sib);

          if (hasClass(sib, 'menu')) {
            sib.classList.remove('open');
            sib.setAttribute('aria-hidden', false);
          }
        });
      } else if (hasClass(elem, ['menu-link'])) {
        canOpen = false;
        elem.classList.remove('open');
        touched.push(elem);
      } else if (hasClass(elem, ['menu-item'])) {
        if (canOpen) {
          elem.classList.add('open');
        } else {
          elem.classList.remove('open');
        }
        touched.push(elem);
      } else if (hasClass(elem, ['block-menu']) || hasClass(elem, ['dropdown-menus'])) {
        elem.classList.add('open');
        touched.push(elem);
      }
    });

    close = close.filter(function (elem) { return touched.indexOf(elem) < 0; });

    close.forEach(function (elem) {
      elem.classList.remove('open');

      if (hasClass(elem, 'menu')) {
        elem.setAttribute('aria-hidden', true);
      }
    });

    return touched.length;
  }

  function clickHandler (e) {
    activateMenu(e.target);

    if (hasClass(e.target, ['menu-link', 'has-submenu'])) {
      e.preventDefault();
      e.stopPropagation();
    }
  }

  function keyupHandler (e) {
    if (e.keyCode === 27 && activateMenu()) {
      e.preventDefault();
      e.stopPropagation();
    }
  }

  Drupal.behaviors.soahMenus = {
    attach: function () {
      var elems = document.querySelectorAll('.dropdown-menus .menu-link.has-submenu');

      for (var c = 0; c < elems.length; c++) {
        elems[c].classList.add('menu-handler');
        elems[c].setAttribute('aria-haspopup', true);

        forSiblings(elems[c], function (sib) {
          if (hasClass(sib, 'menu') && !sib.getAttribute('aria-hidden')) {
            sib.setAttribute('aria-hidden', true);
          }
        });
      }

    }
  };

  document.addEventListener('click', clickHandler);
  document.addEventListener('keyup', keyupHandler);
  Drupal.behaviors.soahMenus.attach();
})();
